using System;
using System.Collections.Generic;
using System.Collections;

namespace Qnomad {

	public static class CollectionExtensions {

		/// <summary>
		/// Compares the contents of two collections independent of item order.
		/// </summary>
		/// <seealso href="http://patforna.blogspot.com/2006/12/comparing-collections-independent-of.html"/>
		/// <typeparam name="T">The collections' item type</typeparam>
		/// <returns>True if both collections contain the same elements (in any order)</returns>
		public static bool EqualsIgnoreOrder<T>( this ICollection<T> one, ICollection<T> two ) {
			if ( one == null || two == null ) //special case
				return one == two;
			else if ( ReferenceEquals( one, two ) ) //same objects
				return true;
			else if ( one.Count != two.Count )
				return false;

			var listOne = new List<T>( one );
			var listTwo = new List<T>( two );

			// Make sure that every object in one is also in two.
			for ( int i = 0; i < listOne.Count; i++ ) {
				T obj = listOne[i];
				if ( !listTwo.Contains( obj ) )
					return false;
				else
					listTwo.Remove( obj );
			}
			return true;
		}
	}
}
