﻿using System;
using System.Collections.Generic;

namespace Qnomad.Outlining
{
	public interface IMoveable<TContainer>
	{
		/// <summary>
		/// In general, can this item be moved? Controls whether or not the DragDropManager allows a drag operation to start.
		/// A drag start will be prevented if this method returns false.
		/// </summary>
		/// <returns>True if a drag operation can begin; false otherwise</returns>
		bool CanMove();

		/// <summary>
		/// Can this item be moved to the specified location? Controls whether or not the DragDropManager
		/// allows the item to be dropped. The proposed location is otherwise valid, but dropping it there
		/// will be disallowed if this method returns false.
		/// </summary>
		/// <param name="newParent">The proposed new parent</param>
		/// <param name="newChildIndex">The proposed new index</param>
		/// <returns>True if the item can be dropped; false otherwise</returns>
		bool CanMove( TContainer newParent, int newChildIndex );

		//void Move( TContainer newParent, int newChildIndex );
	}
}
