using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Input;

namespace Qnomad.Outlining {

	/// <summary>Defines outlining-related routed-commands</summary>
	public static class OutliningCommands {
		private static Type oc = typeof( OutliningCommands );

		public static readonly RoutedUICommand InsertSiblingAfter =
			new RoutedUICommand( "Sibling After", "InsertSiblingAfter", oc );

		public static readonly RoutedUICommand InsertSiblingBefore =
			new RoutedUICommand( "Sibling Before", "InsertSiblingBefore", oc );

		public static readonly RoutedUICommand InsertChild =
			new RoutedUICommand( "Child", "InsertChild", oc );

		public static readonly RoutedUICommand InsertParent =
			new RoutedUICommand( "Parent", "InsertParent", oc );

		public static readonly IList<RoutedUICommand> Inserts = new ReadOnlyCollection<RoutedUICommand>(
			new List<RoutedUICommand> { InsertSiblingAfter, InsertSiblingBefore, InsertChild, InsertParent }
		);

		public static readonly RoutedUICommand NudgeDemote =
			new RoutedUICommand( "Demote", "NudgeDemote", oc );

		public static readonly RoutedUICommand NudgePromote =
			new RoutedUICommand( "Promote", "NudgePromote", oc );

		public static readonly RoutedUICommand NudgeFrontward =
			new RoutedUICommand( "Frontward", "NudgeFrontward", oc );

		public static readonly RoutedUICommand NudgeRearward =
			new RoutedUICommand( "Rearward", "NudgeRearward", oc );

		public static readonly IList<RoutedUICommand> Nudges = new ReadOnlyCollection<RoutedUICommand>(
			new List<RoutedUICommand> { NudgeDemote, NudgePromote, NudgeFrontward, NudgeRearward }
		);

		public static readonly RoutedUICommand EnterSelectMode =
			new RoutedUICommand( "Select/Move Nodes", "EnterSelectMode", oc );

		public static readonly RoutedUICommand EnterEditMode =
			new RoutedUICommand( "Edit Outline Text", "EnterEditMode", oc );

		static OutliningCommands() {
			InsertSiblingAfter.InputGestures.Add( new KeyGesture(Key.Enter, ModifierKeys.None, "Enter") );
			InsertChild.InputGestures.Add( new KeyGesture(Key.Enter, ModifierKeys.Shift, "Shift+Enter") );
			NudgeDemote.InputGestures.Add( new KeyGesture(Key.Tab, ModifierKeys.None, "Tab") );
			NudgePromote.InputGestures.Add( new KeyGesture(Key.Tab, ModifierKeys.Shift, "Shift+Tab") );

			NudgeFrontward.InputGestures.Add( new KeyGesture(Key.Down, ModifierKeys.Control, "Ctrl+Down") );
			NudgeRearward.InputGestures.Add( new KeyGesture(Key.Up, ModifierKeys.Control, "Ctrl+Up") );

			EnterSelectMode.InputGestures.Add( new KeyGesture(Key.Escape, ModifierKeys.None, "Esc") );
			EnterEditMode.InputGestures.Add( new KeyGesture(Key.F2, ModifierKeys.None, "F2") );
		}
	}
}
