﻿using System;
using System.Globalization;
using System.Text.RegularExpressions;
using System.Windows.Data;

namespace Qnomad.Presentation
{
	public class ObjectToClassNameConverter : IValueConverter
	{
		public object Convert( object value, Type targetType, object parameter, CultureInfo culture )
		{
			return value != null ? Regex.Replace( value.GetType().Name, @"(\B[A-Z])", @" $1" ) : null;
		}

		public object ConvertBack( object value, Type targetType, object parameter, CultureInfo culture )
		{
			throw new NotImplementedException();
		}
	}

	/// <summary>
	/// Provides for two-way binding between an Enum property and a boolean value.
	/// </summary>
	public class EnumToBooleanConverter : IValueConverter
	{
		public string TrueValue { get; set; }
		public string FalseValue { get; set; }

		#region IValueConverter Members

		public object Convert( object value, Type targetType, object parameter, CultureInfo culture )
		{
			string trueValue = parameter != null ? (string)parameter : this.TrueValue;

			if ( value == null || trueValue == null )
				return false;
			else
				return value.ToString().Equals( trueValue, StringComparison.InvariantCultureIgnoreCase );
		}

		public object ConvertBack( object value, Type targetType, object parameter, CultureInfo culture )
		{
			string trueValue = parameter != null ? (string)parameter : this.TrueValue;

			if ( value == null )
				return null;
			else if ( trueValue != null && (bool)value )
				return Enum.Parse( targetType, trueValue, true );
			else if ( FalseValue != null && !(bool)value )
				return Enum.Parse( targetType, FalseValue, true );
			else
				return null;
		}

		#endregion
	}
}
