using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using Qnomad.Outlining;

namespace Qnomad.Presentation
{
	public static class WpfExtentions
	{
		#region Tree Operations

		public static IEnumerable<DependencyObject> Ancestors( this DependencyObject startNode )
		{
			return TreeOperations.Ancestors( startNode, WpfTreeNodeAdapter.Instance );
		}

		public static IEnumerable<DependencyObject> AncestorsAndSelf( this DependencyObject startNode )
		{
			return TreeOperations.AncestorsAndSelf( startNode, WpfTreeNodeAdapter.Instance );
		}

		public static IEnumerable<DependencyObject> Descendants( this DependencyObject root )
		{
			return TreeOperations.Descendants( root, WpfTreeNodeAdapter.Instance );
		}

		public static IEnumerable<DependencyObject> DescendantsAndSelf( this DependencyObject root )
		{
			return TreeOperations.DescendantsAndSelf( root, WpfTreeNodeAdapter.Instance );
		}

		public static IEnumerable<DependencyObject> PrecedingSiblings( this DependencyObject context )
		{
			return TreeOperations.PrecedingSiblings( context, WpfTreeNodeAdapter.Instance );
		}

		public static IEnumerable<DependencyObject> FollowingSiblings( this DependencyObject context )
		{
			return TreeOperations.FollowingSiblings( context, WpfTreeNodeAdapter.Instance );
		}

		#endregion Tree Operations

		#region TextBox Extention Methods

		public static int GetInsertionIndexFromPoint( this TextBox textBox, Point point, bool snapToText ) {
			int index = textBox.GetCharacterIndexFromPoint( point, snapToText );
			if ( index == -1 )  return -1;

			Rect startRect = textBox.GetRectFromCharacterIndex( index, false );
			Rect endRect = textBox.GetRectFromCharacterIndex( index, true );

			if ( Math.Abs(point.X - endRect.X) < Math.Abs(point.X - startRect.X) ) {
				index++;
			}
			return index;
		}

		#endregion

		public static IList<ValidationError> GetErrors( this DependencyObject subtree )
		{
			return subtree.DescendantsAndSelf()
				.Where( d => Validation.GetHasError( d ) )
				.SelectMany( d => Validation.GetErrors( d ) )
				.ToList();
		}
	}
}
