﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Qnomad.Outlining;

namespace Qnomad.Presentation
{
	internal class WpfTreeNodeAdapter : ITreeNodeAdapter<DependencyObject>
	{
		#region Object Creation

		private WpfTreeNodeAdapter() { }
		private static WpfTreeNodeAdapter instance;

		public static WpfTreeNodeAdapter Instance
		{
			get
			{
				if ( instance == null )
					instance = new WpfTreeNodeAdapter();
				return instance;
			}
		}

		#endregion Object Creation

		public IEnumerable<DependencyObject> GetSubItems( DependencyObject treeNode )
		{
			for ( int i = 0; i < VisualTreeHelper.GetChildrenCount(treeNode); i++ )
				yield return VisualTreeHelper.GetChild( treeNode, i );
		}

		public bool HasSubItems( DependencyObject treeNode )
		{
			return VisualTreeHelper.GetChildrenCount( treeNode ) > 0;
		}

		/// <summary>
		/// GetParent method that traverses through the visual tree, as well as content elements.
		/// </summary>
		/// <seealso href="http://code.logos.com/blog/2008/02/finding_ancestor_elements_in_w.html"/>
		public DependencyObject GetParent( DependencyObject treeNode )
		{
			if ( treeNode == null ) return null;
			var ce = treeNode as ContentElement;
			if ( ce != null ) {
				DependencyObject parent = ContentOperations.GetParent( ce );
				if ( parent != null ) return parent;

				var fce = ce as FrameworkContentElement;
				return fce != null ? fce.Parent : null;
			}
			return VisualTreeHelper.GetParent( treeNode );
		}
	}
}
