using System;
using System.Collections.Generic;
using System.Reflection;

namespace Qnomad {

	public static class ReflectionExtentions {

		public static object NavigateToOneSide( this object start, string propName ) {
			return GetPropertyValue( start, propName, true );
		}

		public static object TryNavigateToOneSide( this object start, string propName ) {
			return GetPropertyValue( start, propName, false );
		}

		public static ICollection<T> NavigateToManySide<T>( this object start, string propName ) {
			return (ICollection<T>)GetPropertyValue( start, propName, true );
		}

		static object GetPropertyValue( object start, string propName, bool throwsException )
		{
			BindingFlags bf = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
			PropertyInfo pi = start.GetType().GetProperty( propName, bf );

			if ( pi != null )
				pi = pi.DeclaringType.GetProperty( propName, bf );

			if ( pi == null && throwsException )
				throw new InvalidOperationException( "Property does not exist!" );
			else if ( pi == null )
				return null;
			else
				return pi.GetValue( start, null );
		}

		#region 'IsInitialized' Method

		private static MethodInfo isInitialized;

		public static bool IsInitialized<T>( ICollection<T> newCollection ) {
			if ( isInitialized == null ) {
				Type t = Type.GetType( "NHibernate.NHibernateUtil, NHibernate" );
				if ( t != null )
					isInitialized = t.GetMethod( "IsInitialized", BindingFlags.Static | BindingFlags.Public );
			}
			if ( isInitialized != null ) // true if the NHibernate assembly is present
				return (bool)isInitialized.Invoke( null, new object[] { newCollection } );
			else
				return true;
		}

		#endregion

	}
}
