using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;

using Qnomad.BidirectionalAssociations;
using Qnomad.Outlining;

namespace Qnomad.Demos.PresentationModel {

	public class SampleTreeNode : ITreeNode<SampleTreeNode> {

		#region Properties

		private string name;
		private IList<SampleTreeNode> subNodes;
		private SampleTreeNode parentNode;

		public string Name {
			get { return name; }
			set { name = value; }
		}

		public IList<SampleTreeNode> SubNodes {
			get {
				if ( subNodes == null ) {
					subNodes = new ObservableCollection<SampleTreeNode>();
					( (INotifyCollectionChanged)subNodes ).CollectionChanged +=
						new OneToManyAssocSync( this, "ParentNode" ).UpdateManySide;
				}
				return subNodes;
			}
		}

		public SampleTreeNode ParentNode {
			get { return parentNode; }
			set {
				SampleTreeNode oldParentNode = parentNode;
				parentNode = value;
				OneToManyAssocSync.UpdateOneSide( this, oldParentNode, parentNode, "SubNodes" );
			}
		}

		#endregion Properties

		public SampleTreeNode( string name ) { this.name = name; }

		#region ITreeNode Members

		SampleTreeNode IContained<SampleTreeNode, SampleTreeNode>.Parent {
			get { return ParentNode; }
		}

		IList<SampleTreeNode> IContainer<SampleTreeNode, SampleTreeNode>.SubItems {
			get { return SubNodes; }
		}

		bool IContainer<SampleTreeNode, SampleTreeNode>.HasSubItems {
			get { return subNodes != null && subNodes.Count > 0; }
		}

		#endregion

		public override string ToString() {
			return GetType().Name + ": Name=\"" + Name + "\"";
		}

		#region Static Tree Data

		public static SampleTreeNode Root {
			get {
				//Build sample domain-model data:
				SampleTreeNode reptilia = new SampleTreeNode( "Reptilia" );
				SampleTreeNode anapsida = new SampleTreeNode( "Anapsida" );
				SampleTreeNode parapsida = new SampleTreeNode( "Parapsida" );
				SampleTreeNode diapsida = new SampleTreeNode( "Diapsida" );
				SampleTreeNode synapsida = new SampleTreeNode( "Synapsida" );
				reptilia.SubNodes.Add( anapsida );
				reptilia.SubNodes.Add( parapsida );
				reptilia.SubNodes.Add( diapsida );
				reptilia.SubNodes.Add( synapsida );
				anapsida.SubNodes.Add( new SampleTreeNode("Captorhinida") );
				anapsida.SubNodes.Add( new SampleTreeNode("Chelonia") );
				parapsida.SubNodes.Add( new SampleTreeNode("Ichthyosauria") );
				parapsida.SubNodes.Add( new SampleTreeNode("Plesiosauria") );
				SampleTreeNode lepidosauria = new SampleTreeNode( "Lepidosauria" );
				SampleTreeNode archosauria = new SampleTreeNode( "Archosauria" );
				diapsida.SubNodes.Add( lepidosauria );
				diapsida.SubNodes.Add( archosauria );
				lepidosauria.SubNodes.Add( new SampleTreeNode("Sephenodonta") );
				lepidosauria.SubNodes.Add( new SampleTreeNode("Squamata") );
				archosauria.SubNodes.Add( new SampleTreeNode("Crocodylia") );
				archosauria.SubNodes.Add( new SampleTreeNode("Saurischia") );
				archosauria.SubNodes.Add( new SampleTreeNode("Ornithischia") );
				archosauria.SubNodes.Add( new SampleTreeNode("Pterosauria") );
				synapsida.SubNodes.Add( new SampleTreeNode("Pelycosauria") );
				synapsida.SubNodes.Add( new SampleTreeNode("Therapsida") );
				return reptilia;
				//[Source: http://www-biol.paisley.ac.uk/Courses/Tatner/biomedia/units/rept10.htm ]
			}
		}

		#endregion Tree Data

	}
}
