using System;
using System.Collections.Generic;
using Qnomad.PresentationModel;
using Qnomad.Outlining;

namespace Qnomad.Demos.PresentationModel.GUI.Model {

	public sealed class Demo3ItemPM : HierarchicalPM<SampleItem, Demo3ItemPM> {

		#region Object Creation

		/// <summary>Creates a root-level Demo3ItemView.</summary>
		public Demo3ItemPM()
			: base( new Representative( "Root" ), App.SampleListContainer.SampleList, false, false, true )
		{
			foreach ( var itemToAdd in App.SampleListContainer.SampleList )
				CreateInstance( itemToAdd ).PreparePosition( this ).Apply(); //iterate w/o using an index
		}

		/// <summary>
		/// Creates a Demo3ItemView that does not (yet) have any sub-items.
		/// </summary>
		/// <param name="domainItem">The sample item to be represented by this node</param>
		private Demo3ItemPM( SampleItem domainItem ) : base( domainItem, null, false, false, true ) { }

		/// <inheritdoc/>
		protected override Demo3ItemPM CreateInstance( SampleItem domainItem ) {
			return new Demo3ItemPM( domainItem );
		}

		#endregion

		#region 'DesiredPosition' Method

		/// <inheritdoc/>
		protected override HierarchicalPlacementArgs<Demo3ItemPM> DesiredPosition( Demo3ItemPM subtree ) {
			HierarchicalPlacementArgs<Demo3ItemPM> result = null;

			Represented represented = DomainItem as Represented;
			if ( represented != null && represented.RepresentedBy != null ) { //If it's represented...
				Demo3ItemPM representativeWrapper =
					subtree.FindDescendant( represented.RepresentedBy ); //Is the representative also related?
				if ( representativeWrapper != null ) { //If related...
					//representativeWrapper.subItems.Add( itemToAdd ); //then add as sub-item of representative
					result = new HierarchicalPlacementArgs<Demo3ItemPM>( OutliningCommands.InsertChild, representativeWrapper, false );
				}
			}

			Representative representative = DomainItem as Representative;
			if ( representative != null ) { //If it's a representative...
				IList<Demo3ItemPM> itemsRepresented =
					subtree.ItemsRepresented( representative ); //Are any of its represented items present?
				if ( itemsRepresented.Count > 0 ) { //If present, then insert as parent of represented items:
					result = new HierarchicalPlacementArgs<Demo3ItemPM>( OutliningCommands.InsertParent, itemsRepresented, false );
				}
			}

			//subItems.Add( itemToAdd );
			if ( result == null )
				result = new HierarchicalPlacementArgs<Demo3ItemPM>( OutliningCommands.InsertChild, subtree, false );
			return result;
		}

		private IList<Demo3ItemPM> ItemsRepresented( Representative representative ) {
			var result = new List<Demo3ItemPM>();
			foreach ( Demo3ItemPM wrapper in subItems ) {
				Represented represented = wrapper.DomainItem as Represented;
				if ( represented != null && represented.RepresentedBy != null && represented.RepresentedBy.Equals( representative ) )
					result.Add( wrapper );
			}
			return result;
		}

		#endregion

		public Representative RepresentedBy {
			get {
				Represented represented = DomainItem as Represented;
				if ( represented != null ) return represented.RepresentedBy;
				else return null;
			}
			set {
				Represented represented = DomainItem as Represented;
				if ( represented != null ) represented.RepresentedBy = value;
			}
		}
	}
}
