using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;

using Qnomad.Demos.PresentationModel.GUI.Model;
using Qnomad.Outlining;

namespace Qnomad.Demos.PresentationModel.GUI {

	/// <summary>
	/// Interaction logic for Demo3.xaml
	/// </summary>
	public partial class Demo3 : UserControl {
		private Demo3ItemPM rootWrapper;
		
		public Demo3() {
			InitializeComponent();

			//Build sample presentation-model objects:
			rootWrapper = new Demo3ItemPM();
			sampleTreeView.ItemsSource = rootWrapper.SubItems;

			representedByComboBox.ItemsSource = App.SampleListContainer.SampleList;
			ICollectionView cv = CollectionViewSource.GetDefaultView( App.SampleListContainer.SampleList );
			cv.Filter = delegate( object item ) { return item is Representative; };
		}

		private void addButton_Click( object sender, RoutedEventArgs args ) {
			Demo3ItemPM selected = (Demo3ItemPM)sampleTreeView.SelectedItem; //get selected item
			Represented newItem = new Represented( editItemBox.Text, //create new item
				(Representative)representedByComboBox.SelectedItem );

			//Insert new item into domain-model tree structure:
			App.SampleListContainer.SampleList.Add( newItem );

			//Select new item in UI:
			Demo3ItemPM newItemView = rootWrapper.FindDescendant( newItem );
			newItemView.IsSelected = true; //select new item
			newItemView.FocusTreeViewItem( sampleTreeView );
		}

		private void updateButton_Click( object sender, RoutedEventArgs args ) {
			BindingExpression be = editItemBox.GetBindingExpression( TextBox.TextProperty );
			be.UpdateSource(); //update sampleTreeView.SelectedItem.DomainItem.Name with editItemBox.Text's value
			( (Demo3ItemPM)editItemBox.DataContext ).UpdatePosition();
			sampleTreeView.Focus();
		}

		private void deleteButton_Click( object sender, RoutedEventArgs args ) {
			Demo3ItemPM selected = (Demo3ItemPM)sampleTreeView.SelectedItem; //get selected item
			App.SampleListContainer.SampleList.Remove( selected.DomainItem );
			sampleTreeView.Focus();
		}
	}

	[ValueConversion( typeof(Demo3ItemPM), typeof(Boolean) )]
	public class ItemSelectedConverter : IValueConverter {
		public object Convert( object value, Type targetType, object parameter, CultureInfo culture ) {
			return value != null;
		}

		public object ConvertBack( object value, Type targetType, object parameter, CultureInfo culture ) {
			return null;
		}
	}
}
